@@ -27,7 +27,8 @@ import unittest
 # Related third party imports
 
 # Local application/library specific imports
-from harvesters.test.base_harvester import TestHarvesterCoreBase
+from harvesters.test.base_harvester import TestHarvester
+from harvesters.test.base_harvester import BaseVersion
 
 
 class ThreadWithTimeLimit(Thread):
@@ -61,12 +62,12 @@ class ThreadWithTimeLimit(Thread):
                 self._is_running = False
 
 
-class TestTutorials(TestHarvesterCoreBase):
+class TestTutorials(TestHarvester):
 
     def _test_performance_on_image_acquisition(self, sleep_duration=0.0):
         #
         self._logger.info(
-            'Sleep duration: {0} s'.format(sleep_duration)
+            'sleep duration: {0} s'.format(sleep_duration)
         )
 
         # Connect to the first camera in the list.
@@ -75,17 +76,17 @@ class TestTutorials(TestHarvesterCoreBase):
         )
 
         # Then start image acquisition.
-        self.ia.start_acquisition(run_in_background=True)
+        self.ia.start(run_as_thread=True)
 
         # Run the image acquisition thread:
         thread = ThreadWithTimeLimit(
-            worker=self._worker_update_statistics, timeout=5
+            worker=self._worker_update_statistics, timeout=3
         )
         thread.start()
         thread.join()
 
         # Stop image acquisition:
-        self.ia.stop_acquisition()
+        self.ia.stop()
 
         # Destroy the image acquirer:
         self.ia.destroy()
@@ -133,10 +134,10 @@ class TestTutorials(TestHarvesterCoreBase):
         self.ia = self.harvester.create_image_acquirer(0)
 
         #
-        self.ia.start_acquisition(run_in_background=True)
+        self.ia.start(run_as_thread=True)
 
         #
-        nr = 100
+        nr = 3
 
         # Run the image acquisition thread:
         threads = []
@@ -144,7 +145,7 @@ class TestTutorials(TestHarvesterCoreBase):
             threads.append(
                 ThreadWithTimeLimit(
                     worker=self._worker_update_statistics,
-                    timeout=10, sleep=0
+                    timeout=3, sleep=0
                 )
             )
 
@@ -155,11 +156,19 @@ class TestTutorials(TestHarvesterCoreBase):
             thread.join()
 
         #
-        self.ia.stop_acquisition()
+        self.ia.stop()
 
         # Destroy the image acquirer:
         self.ia.destroy()
 
 
+class ThreadWithTimeLimitVersion1(ThreadWithTimeLimit):
+    base_version = BaseVersion.VERSION_1
+
+
+class TestTutorialsVersion1(TestTutorials):
+    base_version = BaseVersion.VERSION_1
+
+
 if __name__ == '__main__':
     unittest.main()
