@@ -520,140 +520,6 @@ component_32bit_formats = [
     'Confidence32f',
 ]
 
-component_2d_formats = [
-    #
-    'Mono8',
-    'Mono10',
-    'Mono12',
-    'Mono14',
-    'Mono16',
-    #
-    'Mono10Packed',
-    'Mono12Packed',
-    #
-    'RGB8',
-    'RGB10',
-    'RGB12',
-    'RGB14',
-    'RGB16',
-    #
-    'BGR8',
-    'BGR10',
-    'BGR12',
-    'BGR14',
-    'BGR16',
-    #
-    'RGBa8',
-    'RGBa10',
-    'RGBa12',
-    'RGBa14',
-    'RGBa16',
-    #
-    'BGRa8',
-    'BGRa10',
-    'BGRa12',
-    'BGRa14',
-    'BGRa16',
-    #
-    'BayerGR8',
-    'BayerGB8',
-    'BayerRG8',
-    'BayerBG8',
-    #
-    'BayerGR10',
-    'BayerGB10',
-    'BayerRG10',
-    'BayerBG10',
-    #
-    'BayerGR12',
-    'BayerGB12',
-    'BayerRG12',
-    'BayerBG12',
-    #
-    'BayerGR16',
-    'BayerRG16',
-    'BayerGB16',
-    'BayerBG16',
-    #
-    'Coord3D_A8',
-    'Coord3D_B8',
-    'Coord3D_C8',
-    'Coord3D_ABC8',
-    'Coord3D_ABC8_Planar',
-    'Coord3D_AC8',
-    'Coord3D_AC8_Planar',
-    'Coord3D_A16',
-    'Coord3D_B16',
-    'Coord3D_C16',
-    'Coord3D_ABC16',
-    'Coord3D_ABC16_Planar',
-    'Coord3D_AC16',
-    'Coord3D_AC16_Planar',
-    'Coord3D_A32f',
-    'Coord3D_B32f',
-    'Coord3D_C32f',
-    'Coord3D_ABC32f',
-    'Coord3D_ABC32f_Planar',
-    'Coord3D_AC32f',
-    'Coord3D_AC32f_Planar',
-    #
-    'Confidence1',
-    'Confidence8',
-    'Confidence16',
-    'Confidence32f',
-    #
-    'Mono10p',
-    'Mono12p',
-    #
-    'Coord3D_A10p',
-    'Coord3D_B10p',
-    'Coord3D_C10p',
-    'Coord3D_A12p',
-    'Coord3D_B12p',
-    'Coord3D_C12p',
-    'Coord3D_ABC10p',
-    'Coord3D_ABC10p_Planar',
-    'Coord3D_ABC12p',
-    'Coord3D_ABC12p_Planar',
-    #
-    'YCbCr422_10p',
-    'YCbCr422_12p',
-    'YCbCr601_422_10p',
-    'YCbCr601_422_12p',
-    'YCbCr709_422_10p',
-    'YCbCr709_422_12p',
-    'YCbCr422_10p_CbYCrY',
-    'YCbCr422_12p_CbYCrY',
-    'YCbCr601_422_10p_CbYCrY',
-    'YCbCr601_422_12p_CbYCrY',
-    'YCbCr709_422_10p_CbYCrY',
-    'YCbCr709_422_12p_CbYCrY',
-    'YCbCr2020_422_10p',
-    'YCbCr2020_422_10p_CbYCrY',
-    'YCbCr2020_422_12p',
-    'YCbCr2020_422_12p_CbYCrY',
-    #
-    'RGBa10p',
-    'RGBa12p',
-    'BGRa10p',
-    'BGRa12p',
-    #
-    'Coord3D_AC10p',
-    'Coord3D_AC10p_Planar',
-    'Coord3D_AC12p',
-    'Coord3D_AC12p_Planar',
-    #
-    'Confidence1p',
-    #
-    'BayerBG10p',
-    'BayerBG12p',
-    'BayerGB10p',
-    'BayerGB12p',
-    'BayerGR10p',
-    'BayerGR12p',
-    'BayerRG10p',
-    'BayerRG12p',
-]
 
 rgb_formats = [
     #
@@ -1363,33 +1229,136 @@ class _10p(_PixelFormat):
             unit_depth_in_bit=10,
             location=location
         )
+    
+    def expand(self, array: numpy.ndarray) -> numpy.ndarray:
+        """Expand the Mono10p format (litte-endian order), where chunks of 5 bytes give 4 pixels.
+        
+        Parameters
+        ----------
+        array : numpy.ndarray
+            One-dimensional array of datatype uint8 representing the raw byte sequence.
+        
+        Returns
+        -------
+        numpy.ndarray
+            One-dimensional array of datatype uint16 representing the unpacked
+            pixels with  10-bit data (values from 0 to 1023).
+        """
+
+        assert array.dtype == numpy.uint8
+
+        bytes_packed = 5  # chunks of 5 bytes
+        # pixels_unpacked = 4  # give 4 pixels
+
+        # The .T-transpose allows receiving into five named variables v0--v4.
+        v0, v1, v2, v3, v4 = array.reshape(
+            array.size // bytes_packed,
+            bytes_packed
+        ).astype(numpy.uint16).T
+
+        """
+        See Figure 6-9 on page 34 of
+        https://www.emva.org/wp-content/uploads/GenICam_PFNC_2_3.pdf
+        
+        Input:           v4       v3        v2        v1        v0
+        
+        Byte:            B4       B3        B2        B1        B0
+                        |........|.. ......|.... ....|...... ..|........|
+                        |........ ..|...... ....|.... ......|.. ........|
+        Pixel:           p3          p2          p1          p0
+        
+        Output:          v3          v2          v1          v0
+        
+        """
+        v0 = numpy.bitwise_or(
+            # all the 8 bits of B0 remain as LSB of p0
+            v0,
+            # 2 LSB from B1 go to MSB of p0
+            numpy.bitwise_and(v1 << 8, 0b1100000000)
+        )
+        v1 = numpy.bitwise_or(
+            # 6 MSB from B1 as LSB of p1
+            v1 >> 2,
+            # 4 LSB from B2 of MSB of p1
+            numpy.bitwise_and(v2 << 6, 0b1111000000)
+        )
+        v2 = numpy.bitwise_or(
+            # 4 MSB from B2 as LSB of p2
+            v2 >> 4,
+            # 6 LSB from B3 as MSB of p2
+            numpy.bitwise_and(v3 << 4, 0b1111110000)
+        )
+        v3 = numpy.bitwise_or(
+            # 2 MSB of B3 as LSB of p3
+            v3 >> 6,
+            # all the 8 bits of B4 as MSB of p3
+            v4 << 2
+        )
+
+        # Stack the four pixels as columns, i.e. one row per chunk, then
+        # flatten to 1D-array
+        return numpy.column_stack((v0, v1, v2, v3)).ravel()
+
+
+class _10p32(_PixelFormat):
+    def __init__(
+            self, symbolic: str = None, nr_components: float = None,
+            location: _Location = None):
+        #
+        super().__init__(
+            symbolic=symbolic,
+            alignment=_Alignment(
+                unpacked=_DataSize.UINT16, packed=_DataSize.UINT8
+            ),
+            nr_components=nr_components,
+            unit_depth_in_bit=10,
+            location=location
+        )
 
     def expand(self, array: numpy.ndarray) -> numpy.ndarray:
+        """
+        Expand the Mono10c3p32 (or RGB10p32) format, where chunks of 4 bytes
+        give 3 pixels.
+        """
         nr_packed = 4
-        nr_unpacked = 3
+        # nr_unpacked = 3
         #
         p1st, p2nd, p3rd, p4th = numpy.reshape(
             array, (array.shape[0] // nr_packed, nr_packed)
         ).astype(numpy.uint16).T
-        #
+        """
+        See Figure 6-6 on page 32 of
+        https://www.emva.org/wp-content/uploads/GenICam_PFNC_2_3.pdf
+        
+        Byte:     B3       B2        B1        B0
+        with     |XX......|.... ....|...... ..|........|
+                 |  ...... ....|.... ......|.. ........|
+        Pixel:      p2          p1          p0
+        
+        """
+
         up1st = numpy.bitwise_or(
-            p1st, numpy.bitwise_and(0x300, p2nd << 8)
+            # all the 8 bits of B0 remain as LSB of p0
+            p1st,
+            # 2 LSB from B1 go to MSB of p0
+            numpy.bitwise_and(0x300, p2nd << 8)
         )
         up2nd = numpy.bitwise_or(
+            # 6 MSB from B1 as LSB of p1
             numpy.bitwise_and(0x3f, p2nd >> 2),
-            numpy.bitwise_and(0x3e0, p3rd << 6)
+            # 4 LSB from B2 of MSB of p1.
+            numpy.bitwise_and(0x3c0, p3rd << 6)
+            # NOTE: 0x3e0 changed to 0x3c0 to make more sense but result is
+            # the same (after the << 6 shift the extra bit with 0x3e0 was
+            # anyway zeroed).
         )
         up3rd = numpy.bitwise_or(
-            numpy.bitwise_and(0x7, p3rd >> 5),
-            numpy.bitwise_and(0x7f8, p4th << 3)
-        )
-        #
-        return numpy.reshape(
-            numpy.concatenate(
-                (up1st[:, None], up2nd[:, None], up3rd[:, None]), axis=1
-            ),
-            nr_unpacked * up1st.shape[0]
+            # 4 MSB from B2 as LSB of p2
+            numpy.bitwise_and(0xf, p3rd >> 4),
+            # 6 LSB from B3 as MSB of p2
+            numpy.bitwise_and(0x3f0, p4th << 4)
         )
+        return numpy.column_stack((up1st, up2nd, up3rd)).ravel()
 
 
 class _12p(_PixelFormat):
@@ -1496,6 +1465,16 @@ class _Mono_10p(_10p):
         )
 
 
+class _Mono_10p32(_10p32):
+    def __init__(self, symbolic: str = None):
+        #
+        super().__init__(
+            symbolic=symbolic,
+            nr_components=1.,
+            location=_Location.MONO
+        )
+
+
 class _Mono_12p(_12p):
     def __init__(self, symbolic: str = None):
         #
@@ -1554,6 +1533,12 @@ class Mono10Packed(_Mono_GroupPacked_10):
         super().__init__(symbolic='Mono10Packed')
 
 
+class Mono10c3p32(_Mono_10p32):
+    def __init__(self):
+        #
+        super().__init__(symbolic='Mono10c3p32')
+
+
 class Mono12Packed(_Mono_GroupPacked_12):
     def __init__(self):
         #
@@ -1846,6 +1831,18 @@ class _LMN444_12p(_12p):
 # ----
 
 
+class RGB10p(_LMN444_10p):
+    def __init__(self):
+        #
+        super().__init__(symbolic='RGB10p')
+
+
+class BGR10p(_LMN444_10p):
+    def __init__(self):
+        #
+        super().__init__(symbolic='BGR10p')
+
+
 class Coord3D_ABC10p(_LMN444_10p):
     def __init__(self):
         #
@@ -1858,16 +1855,28 @@ class Coord3D_ABC10p_Planar(_LMN444_10p):
         super().__init__(symbolic='Coord3D_ABC10p_Planar')
 
 
+class RGB12p(_LMN444_12p):
+    def __init__(self):
+        #
+        super().__init__(symbolic='RGB12p')
+
+
+class BGR12p(_LMN444_12p):
+    def __init__(self):
+        #
+        super().__init__(symbolic=self.__class__.__name__)
+
+
 class Coord3D_ABC12p(_LMN444_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_ABC12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_ABC12p_Planar(_LMN444_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_ABC12p_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2067,157 +2076,157 @@ class _LMNO4444_Unpacked_Uint8_8(_LMNO4444_Unpacked_Uint8):
 class YUV422_8_UYVY(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YUV422_8_UYVY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YUV422_8(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YUV422_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_8(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_8(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_8(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_8_CbYCrY(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_8_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_8_CbYCrY(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_8_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_8_CbYCrY(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_8_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_8(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_8_CbYCrY(_LMN422_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_8_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_10(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_10(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_10(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_10_CbYCrY(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_10_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_10_CbYCrY(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_10_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_10_CbYCrY(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_10_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_10(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_10_CbYCrY(_LMN422_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_10_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_12(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_12(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_12(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_12_CbYCrY(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_12_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_12_CbYCrY(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_12_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_12_CbYCrY(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_12_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_12(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_12_CbYCrY(_LMN422_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_12_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2226,145 +2235,145 @@ class YCbCr2020_422_12_CbYCrY(_LMN422_Unpacked_Uint16_12):
 class YCbCr422_10p(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_10p(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_12p(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_10p(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_10p_CbYCrY(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_10p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_10p_CbYCrY(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_10p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_10p_CbYCrY(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_10p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_10p(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_10p_CbYCrY(_LMN422_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_10p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_12p(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_12p(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr422_12p_CbYCrY(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr422_12p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_422_12p_CbYCrY(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_422_12p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_422_12p_CbYCrY(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_422_12p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_12p(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_422_12p_CbYCrY(_LMN422_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_422_12p_CbYCrY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YUV411_8_UYYVYY(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YUV411_8_UYYVYY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr411_8_CbYYCrYY(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr411_8_CbYYCrYY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr601_411_8_CbYYCrYY(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr601_411_8_CbYYCrYY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr709_411_8_CbYYCrYY(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr709_411_8_CbYYCrYY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr411_8(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr411_8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class YCbCr2020_411_8_CbYYCrYY(_LMN411_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='YCbCr2020_411_8_CbYYCrYY')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa8(_LMNO4444_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa8(_LMNO4444_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2448,73 +2457,73 @@ class _LMNO4444_Unpacked_Uint16_16(_LMNO4444_Unpacked_Uint16):
 class RGBa10(_LMNO4444_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa10(_LMNO4444_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa12(_LMNO4444_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa12(_LMNO4444_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa14(_LMNO4444_Unpacked_Uint16_14):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa14')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa14(_LMNO4444_Unpacked_Uint16_14):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa14')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa16(_LMNO4444_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa16(_LMNO4444_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa10p(_LMNO4444_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa10p(_LMNO4444_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class RGBa12p(_LMNO4444_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='RGBa12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BGRa12p(_LMNO4444_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='BGRa12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2653,61 +2662,61 @@ class _LM44_Unpacked_Float32_32(_LM44_Unpacked_Float32):
 class Coord3D_AC32f(_LM44_Unpacked_Float32_32):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC32f')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC32f_Planar(_LM44_Unpacked_Float32_32):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC32f_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC8(_LM44_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC8_Planar(_LM44_Unpacked_Uint8_8):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC8_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC10p(_LM44_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC10p_Planar(_LM44_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC10p_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC12p(_LM44_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC12p_Planar(_LM44_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC12p_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC16(_LM44_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Coord3D_AC16_Planar(_LM44_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='Coord3D_AC16_Planar')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2792,97 +2801,97 @@ class _Bayer_Unpacked_Uint16_16(_Bayer_Unpacked_Uint16):
 class BayerGR8(_Bayer_Unpacked_Uint8):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG8(_Bayer_Unpacked_Uint8):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB8(_Bayer_Unpacked_Uint8):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG8(_Bayer_Unpacked_Uint8):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG8')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR10(_Bayer_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG10(_Bayer_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB10(_Bayer_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG10(_Bayer_Unpacked_Uint16_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG10')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR12(_Bayer_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG12(_Bayer_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB12(_Bayer_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG12(_Bayer_Unpacked_Uint16_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG12')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR16(_Bayer_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG16(_Bayer_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB16(_Bayer_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG16(_Bayer_Unpacked_Uint16_16):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG16')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 # ----
@@ -2934,97 +2943,97 @@ class _Bayer_12p(_12p):
 class BayerGR10Packed(_Bayer_GroupPacked_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR10Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG10Packed(_Bayer_GroupPacked_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG10Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB10Packed(_Bayer_GroupPacked_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB10Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG10Packed(_Bayer_GroupPacked_10):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG10Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG10p(_Bayer_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB10p(_Bayer_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR10p(_Bayer_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG10p(_Bayer_10p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG10p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR12Packed(_Bayer_GroupPacked_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR12Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG12Packed(_Bayer_GroupPacked_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG12Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB12Packed(_Bayer_GroupPacked_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB12Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG12Packed(_Bayer_GroupPacked_12):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG12Packed')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerBG12p(_Bayer_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerBG12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGB12p(_Bayer_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGB12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerGR12p(_Bayer_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerGR12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class BayerRG12p(_Bayer_12p):
     def __init__(self):
         #
-        super().__init__(symbolic='BayerRG12p')
+        super().__init__(symbolic=self.__class__.__name__)
 
 
 class Dictionary:
@@ -3065,6 +3074,7 @@ class Dictionary:
         # Mono4p(),
         Mono10Packed(),
         Mono10p(),
+        Mono10c3p32(),
         Mono12Packed(),
         Mono12p(),
         Mono14p(),
@@ -3199,13 +3209,17 @@ class Dictionary:
         BayerGR12p(),
         BayerRG10p(),
         BayerRG12p(),
+        RGB10p(),
+        RGB12p(),
+        BGR10p(),
+        BGR12p(),
     ]
 
     def __init__(self):
         #
         super().__init__()
         #
-        for p in self._pixel_formats:
+        for p in Dictionary._pixel_formats:
             print(p)
         pass
 
@@ -3213,11 +3227,16 @@ class Dictionary:
     def get_proxy(cls, symbolic: str):
         #
         _pf = None
-        for pf in cls._pixel_formats:
+        for pf in Dictionary._pixel_formats:
             if symbolic == pf.symbolic:
                 _pf = pf
                 break
         #
         return _pf
 
+    @classmethod
+    def get_pixel_formats(cls):
+        return Dictionary._pixel_formats
 
+
+component_2d_formats = [p.symbolic for p in Dictionary.get_pixel_formats()]
